#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a painting on satin

# (c) Dinasset 2021
#
#   History:
#
#   v1.0: 2021-11: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
FixWH=2000

############################################################################
# Service routines
#---------------------------------------------------------------------------
  
#---------------------------------------- 
# for messages use: erro_msg or info_msg
#---------------------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

#----------------------------------------------- 
# from Original drawable create the Output image
#----------------------------------------------- 
def createWrkimage(inImage, inLayer):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    OrigWH = inLayer.width+inLayer.height    
    
   # define/calculate the scale factor for the (new) output image  
    scale = (FixWH*1.0)/(OrigWH*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size  
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
    # adjust mean values
    adjustmeanSV (newImage, newBG)
    # cut extremes
    pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 1.0,
                                      0.25, 0.75, False)
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
   
    newBG.opacity = 100.0
    
    return(newImage)
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# adjust values and saturation
#---------------------------------------------------------------------------
def adjustmeanSV (inImage, inLayer):
    # decompose HSV
    (HSVI,HL,SL,VL) = pdb.plug_in_decompose (inImage, inLayer, "HSV", True)
    #pdb.gimp_display_new (HSVI)
    HL=HSVI.layers[0]; SL=HSVI.layers[1]; VL=HSVI.layers[2];
    adjustmeanv (HSVI, SL)
    adjustmeanv (HSVI, VL)
    pdb.plug_in_recompose (HSVI, HL)
    return(inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    meanadj=50.0
    # take care of the existing mean in determining the theoretical mean 
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    thmean = (127.5*meanadj/100+meanl*(100.0-meanadj)/100)   
    #info_msg ("thmean="+str(thmean))
    # range for the loops
    minmean = thmean*1.05  
    maxmean = thmean*1.15  
    #info_msg ("target min mean value="+str(minmean))
    #info_msg ("target max mean value="+str(maxmean))
    
    if meanl<minmean:
        count=0
        while meanl<minmean and count<30:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 1.1,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
            count+=1
    elif meanl>maxmean:
        count=0
        while meanl>maxmean and count<30:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 0.9,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
            count+=1
    
    return(inLayer)


############################################################################
# Main
#---------------------------------------------------------------------------

def SketchAndPaint (inImage, inLayer,
                  UserOutWH, 
                  doFlatten):
    
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    OrigWH = inLayer.width+inLayer.height    
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    # create new image to work on
    newImage = createWrkimage (inImage, inLayer)
    width = newImage.width
    height = newImage.height
    Preprocessed = newImage.layers[0]
    Preprocessed.name = "Preprocessed"
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # Apply samj_Edges_And_LIC 2,200,0,0,0,8,8,127,255,0,0,1,20,0.2,0,14,1
    #       samj_Edges_And_LIC 0,200,0,0,0,15,8,127,255,0,1,1,20,0.2,1,14,1
    Sketchparms = '2,200,0,0,0,8,8,127,255,0,0,1,20,0.2,0,14,1'
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -samj_Edges_And_LIC '+Sketchparms+' ')
    Sketch = newImage.layers[0]
    Sketch.mode = LAYER_MODE_GRAIN_MERGE  
    Sketch.opacity = 100.0  
    Sketch.name = "Sketch using samj_Edges_And_LIC"
    
    #pdb.gimp_display_new (newImage)
    #return
    
    pdb.gimp_image_set_active_layer (newImage, Preprocessed)
    # Apply samj_Posterize_B 1,7,1,0,40,30,1,0,1,0,0,1,0,0,0,0,0,0
    Posterparms = '1,7,1,0,40,30,1,0,1,0,0,1,0,0,0,0,0,0'
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -samj_Posterize_B '+Posterparms+' ')
    Poster = newImage.layers[0]
    Poster.mode = LAYER_MODE_GRAIN_MERGE   
    Poster.opacity = 100.0  
    Poster.name = "Poster using samj_Posterize_B"
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = "Outcome" 
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
        
    outcome.name = st+oname+" with a sketch and paint look "
    
    # user required output image size
    if UserOutWH==0: UserOutWH=OrigWH
    if UserOutWH != FixWH:
        uRatio = UserOutWH*1.00/FixWH*1.00
        uWidth = int(width*uRatio)
        uHeight = int(height*uRatio)
        #info_msg ("SCALING")   # not during tests, scaling is slow
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
    
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleGraphics/"
copyrightYear="2021"
desc="Sketch and Paint"

register(
    "SketchAndPaint",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)",
                                   0, (0, 7200, 100)),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    SketchAndPaint,
    menu=menu
)

main()
